# This program is an example of how to create animation in a graphics
# window.  The user can indicate a new location for a circle.  The
# overall change in x and change in y is calculated.  Then the circle
# is moved in very small steps, with a slight pause in between, towards
# the final destination. 

from graphics import *
from time import sleep

FRAMES = 300


def main():
    w = GraphWin("Animate me!", 400, 400)
    w.setBackground("orange")

    current = Point(60,60)
    circ = Circle(current, 50)
    circ.setFill("yellow")
    circ.setWidth(3)
    circ.draw(w)

    message = Text(Point(200,380), "Click mouse on new location for circle")
    message.setSize(15)
    message.draw(w)

    for i in range(3):
        moveTo = w.getMouse()
        message.setText("Watch it move...")
        animate(circ, moveTo)
        message.setText("Click mouse on another new location")

    message.setText("Click once more to quit")
    w.getMouse()


# Input: move the shape object to the Point newPos
# Post-condition: the shape is centered at newPos
def animate( shape, newPos ):
    currPos = shape.getCenter()

    dx = float(newPos.getX() - currPos.getX())
    dy = float(newPos.getY() - currPos.getY())

    for step in range(FRAMES):
        shape.move(dx/FRAMES, dy/FRAMES)
        sleep(0.001)

main()
